Import-Module SQLitePSProvider -ErrorAction Stop

$Noun = "drink"

# If there's already a "data" drive, we'll just use that. Otherwise, load one
if(!(Test-Path data:)) {
   $CreditRoot = Get-Variable PSScriptRoot -ErrorAction SilentlyContinue | ForEach-Object { $_.Value }
   if(!$CreditRoot) {
      $CreditRoot = Split-Path $MyInvocation.MyCommand.Path -Parent
   }   
   Mount-SQLite -name data -dataSource $(Join-Path $CreditRoot "data.sqlite")
}

# We need a table or two for this ... 
if(!(Test-Path data:\Credit)) {
   New-Item data:\Credit -Value "helper TEXT NOT NULL, user TEXT NOT NULL, credits INTEGER DEFAULT 0, PRIMARY KEY(helper, user)"
}


function Get-Credit {
   param(
      # The user who's credit we're checking
      [string]$helper,

      # Optionally, only show credit from a certain (set of) users
      # Supports wildcards
      [string[]]$user='%'
   )
   process {
      foreach($u in $user -replace '\*','%') {
         $u = $u -replace '[^A-Za-z0-9_\^`\|\[\]\%-]'
         $helper = $helper -replace '[^A-Za-z0-9_\^`\|\[\]-]'
         $credits = invoke-item data: -sql "select * from Credit 
                                           where lower(helper) = lower('$helper')
                                           and lower(user) like lower('$u')"

         if(!$PSBoundParameters.ContainsKey('user')) {
            $sum = $credits | measure-object credits -sum
            $credits = [PSCustomObject]@{credits = $sum.sum; users = $sum.count}
         }

         # I don't normally like outputting formatted strings, but this is for IRC
         if(!$credits) {
            "$helper is owed no $noun"
         } else {
            foreach($credit in $credits) {
               $message = if($credit.credits -eq 1) {
                  "$helper is owed $($credit.credits) $noun"
               } else {
                  "$helper is owed $($credit.credits) ${noun}s"
               }

               if(!$credit.users) {
                   $message + " by $($credit.user)"
               } else {
                   $message + " by $($credit.users) users"
               }
            }
         }
      }
   }
}

function Add-Credit {
   param(
      # The user who's credit we're updating.
      # If this is blank, no upate happens
      [Parameter(ValueFromPipeline=$true, Position=0)]
      [string[]]$helper,

      # How much to increment the credit (defaults to 1, allows up to 6)
      [Parameter(ValueFromPipeline=$true, Position=1)]
      [ValidateRange(-1,6)]
      [int]$count = 1
   )
   begin {
      $user = $Nick -replace '[^A-Za-z0-9_\^`\|\[\]\*-]'
   }
   process {
      foreach($help in $helper) {
         # No giving credit to yourself
         if($help -eq $user) { continue }

         # No * here, because we don't want that in the database
         $help = $help -replace '[^A-Za-z0-9_\^`\|\[\]-]'
         invoke-item data: -sql "INSERT OR REPLACE INTO Credit (helper, user, credits) 
                                 VALUES ( 
                                    lower('$help'),
                                    lower('$user'),
                                    abs(COALESCE(
                                       (SELECT credits FROM Credit 
                                        WHERE lower(helper) = lower('$help')
                                        AND lower(user) = lower('$user')) + $count, $count))
                                 )"
      }
      Get-Credit -Helper $(if($Helper){$helper}else{$user}) -Noun $Noun
   }
}

Set-Alias "Add-Credit" "Add-Credit"

Set-Alias "beer" "Add-Credit"
Set-Alias "Add-Beer" "Add-Credit"
Set-Alias "Buy-Beer" "Add-Credit"
Set-Alias "Get-Beer" "Get-Credit"

Set-Alias "drink" "Add-Credit"
Set-Alias "Add-Drink" "Add-Credit"
Set-Alias "Buy-Drink" "Add-Credit"
Set-Alias "Get-Drink" "Get-Credit"

Export-ModuleMember -Function * -Alias *